<?php

/*
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 */

class Note {

    private $noteid;
    private $ownerid;
    private $content = "";
    private $title = "";
    private $modified = "";
    private $color = "FFFFFF";

    /**
     * Create a new Note object.
     * @param string $content Note content in Markdown
     * @param string $color Hex color "RRGGBB"
     * @param int $ownerid The owner's user ID
     */
    public function __construct(string $content, string $color = "FFFFFF", int $ownerid = null, int $noteid = null) {
        $this->content = $content;
        $this->color = $color;
        $this->ownerid = $ownerid;
        $this->noteid = $noteid;
    }

    /**
     * Load a note from the database and return it.
     * @global type $database
     * @param int $noteid
     * @return \Note
     * @throws NoSuchNoteException When the note ID isn't found.
     */
    public static function loadNote(int $noteid): Note {
        global $database;

        if (!$database->has('notes', ['noteid' => $noteid])) {
            throw new NoSuchNoteException();
        }

        $notedata = $database->get('notes', ['noteid', 'ownerid', 'color', 'content', 'title', 'modified'], ['noteid' => $noteid]);

        $note = new Note($notedata['content'], $notedata['color'], $notedata['ownerid'], $notedata['noteid']);
        $note->setTitle(is_null($notedata['title']) ? "" : $notedata['title']);
        $note->setModified(is_null($notedata['modified']) ? date("Y-m-d H:i:s") : $notedata['modified']);

        return $note;
    }

    /**
     * Save the note to the database.
     * @global type $database
     * @param bool $saveas If true, save the note under a new ID.  Forced to
     * true if the current note ID is missing or invalid.
     * @return int The database ID of the saved note
     * @throws Exception If there is no note owner set.
     */
    public function saveNote(bool $saveas = false): int {
        global $database;

        $data = [
            'ownerid' => $this->getOwnerID(),
            'color' => $this->getColor(),
            'content' => $this->getText(),
            'title' => $this->getTitle(),
            'modified' => $this->getModified()
        ];

        // We can't UPDATE the database, so use save as for INSERT
        if (empty($this->noteid) || !$database->has('notes', ['noteid' => $this->noteid])) {
            $saveas = true;
        }

        if (empty($this->ownerid)) {
            throw new Exception("No owner set.");
        }

        if ($saveas) {
            $database->insert('notes', $data);
            $this->noteid = $database->id();
        } else {
            $database->update('notes', $data, ['noteid' => $this->noteid]);
        }

        return $this->noteid;
    }

    /**
     * Delete this note from the database.
     * @global type $database
     */
    public function deleteNote() {
        global $database;

        $database->delete('notes', ['noteid' => $this->noteid]);
    }

    /**
     * Get the Markdown content of the note.
     * @return string
     */
    public function getText(): string {
        return $this->content;
    }

    /**
     * Get the HTML render of the note.
     * @param bool $fragment Get just the HTML content, instead of a whole HTML5 file
     * @return string
     */
    public function getHTML(bool $fragment = true): string {
        $parsedown = new Parsedown;
        $parsedown->setSafeMode(true);
        return $parsedown->text($this->content);
    }

    /**
     * Get the note ID.
     * @return int
     */
    public function getID(): int {
        return $this->noteid;
    }

    /**
     * Get the note color as RRGGBB hex.
     * @return string
     */
    public function getColor(): string {
        return $this->color;
    }

    /**
     * Get the owner UID
     * @return int
     */
    public function getOwnerID(): int {
        return $this->ownerid;
    }

    /**
     * Get the owner as a User object
     * @return \User
     */
    public function getOwner(): User {
        return new User($this->ownerid);
    }

    /**
     * Get the note title
     * @return string
     */
    public function getTitle(): string {
        global $Strings;
        $title = $this->title;
        if (empty($title)) {
            if (empty($this->getText())) {
                $title = $Strings->get("New note", false);
            } else {
                $title = explode("\n", $this->getText())[0];
            }
        }
        return $title;
    }

    /**
     * Get the last modified date/time as "Y-m-d H:i:s"
     * @return string
     */
    public function getModified(): string {
        if (empty($this->modified)) {
            return date("Y-m-d H:i:s");
        }
        return date("Y-m-d H:i:s", strtotime($this->modified));
    }

    /**
     * Set the note content
     * @param string $markdown
     */
    public function setText(string $markdown) {
        $this->content = $markdown;
    }

    /**
     * Set the note color to a hex string
     * @param string $color "RRGGBB"
     */
    public function setColor(string $color) {
        $this->color = $color;
    }

    /**
     * Set the owner by UID
     * @param int $uid
     */
    public function setOwnerID(int $uid) {
        $this->ownerid = $uid;
    }

    /**
     * Set the owner
     * @param User $user
     */
    public function setOwner(User $owner) {
        $this->ownerid = $owner->getUID();
    }

    /**
     * Set the note title
     * @param string $title
     */
    public function setTitle(string $title) {
        $this->title = $title;
    }

    /**
     * Set the last modified date/time
     * @param string $datetime
     */
    public function setModified(string $datetime) {
        $this->modified = date("Y-m-d H:i:s", strtotime($datetime));
    }

    /**
     * Get this note as an array.
     * @return string
     */
    public function toArray(): array {
        $owner = new User($this->ownerid);
        $arr = [
            'noteid' => $this->getID(),
            'color' => $this->getColor(),
            'content' => $this->getText(),
            'title' => $this->getTitle(),
            'modified' => $this->getModified(),
            'owner' => [
                'uid' => $owner->getUID(),
                'username' => $owner->getUsername(),
                'name' => $owner->getName()
            ]
        ];

        return $arr;
    }

    /**
     * Get an array suitable for returning via the Nextcloud Notes API
     * https://github.com/nextcloud/notes/wiki/Notes-0.2#get-a-note
     * @return array
     */
    public function toNextcloud(): array {
        return [
            "id" => $this->getID(),
            "modified" => strtotime($this->getModified()),
            "title" => $this->getTitle(),
            "category" => null,
            "content" => $this->getText(),
            "favorite" => false
        ];
    }

    /**
     * Check whether the given User has read access to this note.
     * @param User $user
     */
    public function hasReadAccess(User $user): bool {
        if ($this->getOwnerID() == $user->getUID()) {
            return true;
        }
        return false;
    }

    /**
     * Check whether the given User has write access to this note.
     * @param User $user
     */
    public function hasWriteAccess(User $user): bool {
        if ($this->getOwnerID() == $user->getUID()) {
            return true;
        }
        return false;
    }

    /**
     * Read an array with the structure from toArray().
     * @param array $arr
     * @return \Note A Note constructed from the array data.
     */
    public static function fromArray(array $arr): Note {
        $note = new Note($arr['content'], $arr['color'], $arr['owner']['uid'], $arr['noteid']);
        $note->setTitle($arr['title']);
        $note->setModified($arr['modified']);
        return $note;
    }

}
